﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Data;
using System.Configuration;

namespace MsSqlServices
{
    public class ActivityRecordServices : IServices.IActivityRecordServices, Chocolate.Common.Interface.IDataToEntity<Model.All>
    {
        /// <summary>
        /// 通过活动id查找活动记录，使用自己建立ORM的方式
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public List<Model.All> SelectWhereId(int id)
        {
            using (SqlConnection conn = new SqlConnection())
            {
                conn.ConnectionString = ConfigurationManager.ConnectionStrings["mssql"].ConnectionString;
                conn.Open();
                SqlCommand cmd = conn.CreateCommand();
                string sql = @"SELECT   Activitys.Id AS ActivityId, Activitys.Name AS ActivityName, ActivityRecords.Id AS ActivityRecordId, 
                            ActivityRecords.MemberId, ActivityRecords.ActivityId AS Expr1, Members.Id AS MemberId, 
                            Members.Name AS MemberName, Members.SexId, Members.AreaId, Members.Email, Sexes.Id AS SexId, 
                            Sexes.Name AS SexName, Areas.Id AS AreaId, Areas.Name AS AreaName
                            FROM  ActivityRecords     INNER JOIN
                            Activitys ON ActivityRecords.ActivityId = Activitys.Id INNER JOIN
                            Members ON ActivityRecords.MemberId = Members.Id INNER JOIN
                            Sexes ON Members.SexId = Sexes.Id INNER JOIN
                            Areas ON Members.AreaId = Areas.Id
                            WHERE   (Activitys.Id = @Id)
                            ORDER BY ActivityRecordId DESC";
                cmd.CommandText = sql;
                List<SqlParameter> parameters = new List<SqlParameter>();
                parameters.Add(new SqlParameter("@Id", id));
                cmd.Parameters.AddRange(parameters.ToArray());
                cmd.CommandType = CommandType.Text;
                SqlDataReader reader = cmd.ExecuteReader();
                //将sqldatareader转成实体集合
                List<Model.All> activityRecords = ConvertToList(reader);
                return activityRecords;
            }
        }

        /// <summary>
        /// 通过活动id查找活动记录，使用Entity Framework 4的方式
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public List<MsSqlModel.All> SelectWhereIdEntity4(int id)
        {
            using (MsSqlModel.ModelEntities db = new MsSqlModel.ModelEntities())
            {
                var sql = from activityRecord in db.ActivityRecords
                          join member in db.Members
                          on activityRecord.MemberId equals member.Id
                          join sex in db.Sexes
                          on member.SexId equals sex.Id
                          join area in db.Areas
                          on member.AreaId equals area.Id
                          join activity in db.Activitys
                          on activityRecord.ActivityId equals activity.Id
                          where activity.Id == (long)id
                          orderby activityRecord.Id descending
                          select new MsSqlModel.All()
                          {
                              ActivityId = activity.Id,
                              ActivityName = activity.Name,
                              ActivityRecordId = activityRecord.Id,
                              AreaId = area.Id,
                              AreaName = area.Name,
                              MemberEmail = member.Email,
                              MemberId = member.Id,
                              MemberName = member.Name,
                              SexId = sex.Id,
                              SexName = sex.Name

                          };
                return sql.ToList<MsSqlModel.All>();
            }
        }

        /// <summary>
        /// 使用Entity Framework 4插入单个对象
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public int Insert(MsSqlModel.ActivityRecord data)
        {
            using (MsSqlModel.ModelEntities db = new MsSqlModel.ModelEntities())
            {
                db.ActivityRecords.AddObject(data);
                return db.SaveChanges();
            }

        }

        /// <summary>
        /// 使用Entity Framework 批量插入对象
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public int Insert(List<MsSqlModel.ActivityRecord> data)
        {
            using (MsSqlModel.ModelEntities db = new MsSqlModel.ModelEntities())
            {
                foreach (MsSqlModel.ActivityRecord o in data)
                {
                    db.ActivityRecords.AddObject(o);
                }
                return db.SaveChanges();
            }
        }

        #region 实现Chocolate.Common.Interface.IDataToEntity<T>集合，将数据转成实体
        /// <summary>
        /// 将DataTable数据集转成相应的实体集合
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public List<Model.All> ConvertToList(System.Data.DataTable data)
        {
            List<Model.All> activityRecords = new List<Model.All>();
            System.Data.DataTableReader reader = data.CreateDataReader();
            for (int i = 0; i < data.Rows.Count; i++)
            {
                if (reader.Read())
                {
                    Model.All o = new Model.All();
                    o.Activity.Id = Convert.ToInt64(reader["ActivityId"]);
                    o.Activity.Name = Convert.ToString(reader["ActivityName"]);
                    o.ActivityRecord.Id = Convert.ToInt64(reader["ActivityRecordId"]);
                    o.ActivityRecord.MemberId = Convert.ToInt64(reader["MemberId"]);
                    o.ActivityRecord.ActivityId = Convert.ToInt64(reader["Expr1"]);
                    o.Member.Id = Convert.ToInt64(reader["MemberId"]);
                    o.Member.Name = Convert.ToString(reader["MemberName"]);
                    o.Member.SexId = Convert.ToInt64(reader["SexId"]);
                    o.Member.AreaId = Convert.ToInt64(reader["AreaId"]);
                    o.Member.Email = Convert.ToString(reader["Email"]);
                    o.Sex.Id = Convert.ToInt64(reader["SexId"]);
                    o.Sex.Name = Convert.ToString(reader["SexName"]);
                    o.Area.Id = Convert.ToInt64(reader["AreaId"]);
                    o.Area.Name = Convert.ToString(reader["AreaName"]);
                    activityRecords.Add(o);
                }
            }
            return activityRecords;
        }

        /// <summary>
        /// 将SqlDataReader数据转成相应的实体集合
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public List<Model.All> ConvertToList(System.Data.SqlClient.SqlDataReader data)
        {
            System.Data.DataTable table = new System.Data.DataTable();
            table.Load(data);
            return ConvertToList(table);
        }

        public List<Model.All> ConvertToList(DataTableReader data)
        {
            throw new NotImplementedException();
        }

        public List<Model.All> ConvertToList(DataRow row)
        {
            throw new NotImplementedException();
        }
        #endregion
    }
}
